--------------------------------------------------------------------------------
-- Company:    Digilent inc
-- Engineer:   Kovacs Laszlo - Attila
--
-- Create Date:    09:39:11 03/26/07
-- Design Name:    
-- Module Name:    DCmotorHB3 - Behavioral
-- Project Name:   
-- Target Device:  X-Board, PModHB3
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity HB3 is
    Port ( 
      Clk      : in  std_logic;

      DataIn   : in  std_logic_vector(7 downto 0);
      DataOut  : out std_logic_vector(7 downto 0);
      LowHigh  : in  std_logic;
      Read     : in  std_logic;
      Write    : in  std_logic;
      DataRdy  : out std_logic;

      HB3Dir   : out std_logic;
      HB3En    : out std_logic);
end HB3;

architecture Behavioral of HB3 is

-- freqency of the PWM
signal pwmfreq : std_logic_vector(7 downto 0) := 
   conv_std_logic_vector(50, 8);

-- pulse length
signal pwmlen : std_logic_vector(6 downto 0) := 
   conv_std_logic_vector(25, 7);

-- direction 
signal dir : std_logic := '0';

-- counted for the frequency and pulse length
signal cnt  : std_logic_vector(7 downto 0) := x"00";

-- refresh and update is used for the new configuration to be loaded 
-- at the end of the PWM cycle to finish the previous configuration and 
-- to not change the direction while it is enabled
signal refresh, update  : std_logic;

begin
   
   DataOut <= pwmfreq when LowHigh = '0' else dir&pwmlen;

   process(Clk)
   begin
      if rising_edge(Clk) then
         
         -- on Write set the flag for the configuration update
         -- and clear when it is refreshed
         if Write = '1' then
            update <= '1';
         elsif refresh = '1' then
            update <= '0';
         end if;

         -- signal for the EPP module that the data is ready:
         --  immediately when a read is performed
         --  on write, only after the configuration is refreshed
         if Read = '1' or (update = '1' and refresh = '1') then
            DataRdy <= '1';
         else
            DataRdy <= '0';         
         end if;
         
         -- load the configuration on refresh if it has to be updated
         if update = '1' and refresh = '1' then
            if LowHigh = '0' then
               pwmfreq <= DataIn;
            else
               pwmlen <= DataIn(6 downto 0);
               dir <= DataIn(7);
            end if;
         end if;
      end if;
   end process;


   HB3Dir <= dir;
   
   -- end of the PWM cycle
   refresh <= '1' when cnt = pwmfreq - '1' else '0';

   process(Clk)
   begin
      if rising_edge(Clk) then
         
         -- start next PWM cycle and count
         if refresh = '1' then
            cnt <= x"00";
         else
            cnt <= cnt + '1';
         end if;
         
         -- deactivate enable when pulse length reached
         -- activate when the PWM cycle is started
         if cnt = '0'&pwmlen then
            HB3En <= '0';
         elsif cnt = x"00" then
            HB3En <= '1';            
         end if;   
      end if;
   end process;

end Behavioral;
